@@ -9,11 +9,11 @@ Test::Valgrind::Parser - Base class for Test::Valgrind parsers.
 
 =head1 VERSION
 
-Version 1.14
+Version 1.18
 
 =cut
 
-our $VERSION = '1.14';
+our $VERSION = '1.18';
 
 =head1 DESCRIPTION
 
@@ -55,9 +55,10 @@ sub args { }
 
 =head2 C<parse>
 
-    $tvp->parse($session, $fh);
+    my $aborted = $tvp->parse($session, $fh);
 
-Parse the output of the C<valgrind> process attached to the session C<$session> received through the filehandle C<$fh>.
+Parses the output of the C<valgrind> process attached to the session C<$session> received through the filehandle C<$fh>.
+Returns true when the output indicates that C<valgrind> has aborted.
 
 This method must be implemented when subclassing.
 
@@ -96,7 +97,7 @@ You can find documentation for this module with the perldoc command.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2009,2010,2011,2013 Vincent Pit, all rights reserved.
+Copyright 2009,2010,2011,2013,2015 Vincent Pit, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
 
