@@ -9,11 +9,11 @@ Test::Valgrind::Tool::memcheck - Run an analysis through the memcheck tool.
 
 =head1 VERSION
 
-Version 1.14
+Version 1.18
 
 =cut
 
-our $VERSION = '1.14';
+our $VERSION = '1.18';
 
 =head1 DESCRIPTION
 
@@ -58,7 +58,7 @@ sub new {
 
  my %args = @_;
 
- my $callers = delete $args{callers} || 12;
+ my $callers = delete $args{callers} || 50;
  $callers =~ s/\D//g;
 
  my $self = bless $class->Test::Valgrind::Tool::new(%args), $class;
@@ -97,8 +97,10 @@ sub parser_class {
            ? 'Test::Valgrind::Parser::Suppressions::Text'
            : 'Test::Valgrind::Parser::XML::Twig';
 
- local $@;
- eval "require $class";
+ {
+  local $@;
+  eval "require $class; 1" or die $@;
+ }
 
  return $class;
 }
@@ -114,8 +116,12 @@ This tool emits C<Test::Valgrind::Tool::memcheck::Report> object reports in anal
 sub report_class {
  my ($self, $session) = @_;
 
- $session->do_suppressions ? 'Test::Valgrind::Report::Suppressions'
-                           : 'Test::Valgrind::Tool::memcheck::Report'
+ if ($session->do_suppressions) {
+  require Test::Valgrind::Parser::Suppressions::Text;
+  return 'Test::Valgrind::Report::Suppressions';
+ } else {
+  return 'Test::Valgrind::Tool::memcheck::Report';
+ }
 }
 
 sub args {
@@ -131,7 +137,7 @@ sub args {
   '--error-limit=yes',
  );
 
- push @args, '--track-origins=yes' if  $sess->version ge '3.4.0'
+ push @args, '--track-origins=yes' if  $sess->version >= '3.4.0'
                                    and not $sess->do_suppressions;
 
  push @args, $self->SUPER::args(@_);
@@ -162,7 +168,7 @@ You can find documentation for this module with the perldoc command.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2009,2010,2011,2013 Vincent Pit, all rights reserved.
+Copyright 2009,2010,2011,2013,2015 Vincent Pit, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
 
@@ -174,7 +180,7 @@ package Test::Valgrind::Tool::memcheck::Report;
 
 use base qw<Test::Valgrind::Report>;
 
-our $VERSION = '1.14';
+our $VERSION = '1.18';
 
 my @kinds = qw<
  InvalidFree
